package javax.security.auth.kerberos;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.security.PermissionCollection;
import java.security.Permission;
import java.util.StringTokenizer;
import java.io.Serializable;
import java.security.BasicPermission;

public final class DelegationPermission extends BasicPermission implements Serializable
{
    private static final long serialVersionUID = 883133252142523922L;
    private transient String subordinate;
    private transient String service;
    
    public DelegationPermission(String name) {
        super(name);
        this.init(name);
    }
    
    public DelegationPermission(String name, String actions) {
        super(name, actions);
        this.init(name);
    }
    
    private void init(String s) {
        if (!s.startsWith("\"")) {
            throw new IllegalArgumentException("service principal [" + s + "] syntax invalid: improperly quoted");
        }
        StringTokenizer stringTokenizer = new StringTokenizer(s, "\"", false);
        this.subordinate = stringTokenizer.nextToken();
        if (stringTokenizer.countTokens() == 2) {
            stringTokenizer.nextToken();
            this.service = stringTokenizer.nextToken();
        }
        else if (stringTokenizer.countTokens() > 0) {
            throw new IllegalArgumentException("service principal [" + stringTokenizer.nextToken() + "] syntax invalid: improperly quoted");
        }
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof DelegationPermission)) {
            return false;
        }
        DelegationPermission delegationPermission = (DelegationPermission)permission;
        return this.subordinate.equals(delegationPermission.subordinate) && this.service.equals(delegationPermission.service);
    }
    
    @Override
    public boolean equals(Object o) {
        return o == this || (o instanceof DelegationPermission && this.implies((DelegationPermission)o));
    }
    
    @Override
    public int hashCode() {
        return this.getName().hashCode();
    }
    
    @Override
    public PermissionCollection newPermissionCollection() {
        return new KrbDelegationPermissionCollection();
    }
    
    private synchronized void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
    }
    
    private synchronized void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.init(this.getName());
    }
}
